import 'dart:io';
import 'dart:async';

import 'package:dio/dio.dart';
import 'package:yuro_extension/yuro_extension.dart' show MapExt, CryptoExt;

class RetryInterceptor extends Interceptor {
  final Dio dio;
  final int retries;
  final Duration interval;

  RetryInterceptor(this.dio, this.retries, this.interval);

  static Map<String, int> _retryPool = {};

  @override
  Future onRequest(RequestOptions options) async => options;

  @override
  Future onResponse(Response response) async => response;

  @override
  Future onError(DioError err) async {
    if (_shouldRetry(err)) {
      return _retry(err.request);
    }
    return err;
  }

  bool _shouldRetry(DioError err) {
    bool retry = false;
    switch (err.type) {
      case DioErrorType.CONNECT_TIMEOUT:
      case DioErrorType.SEND_TIMEOUT:
      case DioErrorType.RECEIVE_TIMEOUT:
        retry = true;
        break;
      default:
        retry = err.error != null && err.error is SocketException;
        break;
    }
    String requestSign = _signParam(err.request);
    int times = _retryPool[requestSign.toMd5()] ?? 0;
    if (retry && times <= retries) {
      return true;
    } else {
      if (_retryPool.containsKey(requestSign)) {
        _retryPool.remove(requestSign);
      }
      if (times > retries) {
        // Logger.d('$requestSign, The number of retries has been exhausted');
      }
      return false;
    }
  }

  String _signParam(RequestOptions options) {
    var sb = StringBuffer(options.path)..write('?');
    if (options.queryParameters != null && options.queryParameters.isNotEmpty) {
      sb.write(options.queryParameters.signStr());
    }
    if (options.data != null && (options.data as Map<String, dynamic>).isNotEmpty) {
      sb.write((options.data as Map<String, dynamic>).signStr());
    }
    if (options.extra != null && options.extra.isNotEmpty) {
      sb.write(options.extra.signStr());
    }
    return sb.toString();
  }

  Future<Response> _retry(RequestOptions options) {
    String requestSign = _signParam(options);
    int times = _retryPool[requestSign.toMd5()] ?? 0;
    // Yuro.log.d('$requestSign will ${++times}\' retry after ${interval.inSeconds} seconds');
    _retryPool[requestSign.toMd5()] = times;
    return Future.delayed(
        interval,
        () => dio.request(
              options.path,
              queryParameters: options.queryParameters,
              data: options.data,
              cancelToken: options.cancelToken,
              options: options,
              onReceiveProgress: options.onReceiveProgress,
              onSendProgress: options.onSendProgress,
            ));
  }
}
